@@ -516,7 +516,7 @@ impl ExprWorker<'_> {
                         }
                         ast::DestructuringItem::Named(named) => {
                             let key = Decl::var(named.name()).into();
-                            let val = self.check_pattern_expr(named.expr());
+                            let val = self.check_pattern(named.pattern());
                             names.push((key, val));
                         }
                         ast::DestructuringItem::Spread(spreading) => {
@@ -525,14 +525,17 @@ impl ExprWorker<'_> {
                             } else {
                                 Decl::spread(spreading.span()).into()
                             };
+                            let pattern = Pattern::Expr(Expr::Star).into();
 
                             if inputs.is_empty() {
-                                spread_left =
-                                    Some((decl, self.check_pattern_expr(spreading.expr())));
+                                spread_left = Some((decl.clone(), pattern));
                             } else {
-                                spread_right =
-                                    Some((decl, self.check_pattern_expr(spreading.expr())));
+                                spread_right = Some((decl.clone(), pattern));
                             }
+
+                            self.resolve_as(Decl::as_def(&decl, None));
+                            self.scope_mut()
+                                .insert_mut(decl.name().clone(), decl.into());
                         }
                     }
                 }
@@ -751,7 +754,7 @@ impl ExprWorker<'_> {
         src: &str,
         is_import: bool,
     ) -> Option<Expr> {
-        let fid = resolve_id_by_path(&self.ctx.world, self.fid, src)?;
+        let fid = resolve_id_by_path(&self.ctx.world(), self.fid, src)?;
         let name = Decl::calc_path_stem(src);
         let module = Expr::Decl(Decl::module_with_name(name.clone(), fid).into());
 
@@ -1230,8 +1233,8 @@ impl ExprWorker<'_> {
         }
 
         let scope = match mode {
-            InterpretMode::Math => self.ctx.world.library.math.scope(),
-            InterpretMode::Markup | InterpretMode::Code => self.ctx.world.library.global.scope(),
+            InterpretMode::Math => self.ctx.world().library.math.scope(),
+            InterpretMode::Markup | InterpretMode::Code => self.ctx.world().library.global.scope(),
             _ => return (None, None),
         };
 
@@ -1244,7 +1247,7 @@ impl ExprWorker<'_> {
         }
 
         if name.as_ref() == "std" {
-            let val = Ty::Value(InsTy::new(self.ctx.world.library.std.read().clone()));
+            let val = Ty::Value(InsTy::new(self.ctx.world().library.std.read().clone()));
             return (None, Some(val));
         }
 
