@@ -30,7 +30,7 @@ use sync_ls::{LspClient, TypedLspClient};
 use tinymist_project::vfs::{FileChangeSet, MemoryEvent};
 use tinymist_query::analysis::{Analysis, LspQuerySnapshot, PeriscopeProvider};
 use tinymist_query::{
-    CheckRequest, CompilerQueryRequest, DiagnosticsMap, LocalContext, SemanticRequest,
+    CheckRequest, CompilerQueryRequest, DiagnosticsMap, LocalContext, SemanticRequest, GLOBAL_STATS,
 };
 use tinymist_render::PeriscopeRenderer;
 use tinymist_std::{error::prelude::*, ImmutPath};
@@ -174,6 +174,7 @@ impl ServerState {
                 allow_overlapping_token: const_config.tokens_overlapping_token_support,
                 allow_multiline_token: const_config.tokens_multiline_token_support,
                 remove_html: !config.support_html_in_markdown,
+                support_client_codelens: true,
                 extended_code_action: config.extended_code_action,
                 completion_feat: config.completion.clone(),
                 color_theme: match config.color_theme.as_deref() {
@@ -235,6 +236,7 @@ impl ServerState {
             CompileServerOpts {
                 handler: compile_handle,
                 export_target: config.export_target,
+                syntax_only: config.syntax_only,
                 ignore_first_sync: true,
             },
         );
@@ -549,7 +551,7 @@ impl CompileHandlerImpl {
         if !should_lint {
             let enc = self.analysis.position_encoding;
             let diagnostics =
-                tinymist_query::convert_diagnostics(art.world(), art.diagnostics(), enc);
+                tinymist_query::convert_diagnostics(art.graph.clone(), art.diagnostics(), enc);
 
             log::trace!("notify diagnostics({dv:?}): {diagnostics:#?}");
 
@@ -561,8 +563,7 @@ impl CompileHandlerImpl {
             let editor_tx = self.editor_tx.clone();
             let analysis = self.analysis.clone();
             spawn_cpu(move || {
-                let world = snap.world().clone();
-                let mut ctx = analysis.enter(world);
+                let mut ctx = analysis.enter(snap.graph.clone());
 
                 // todo: check all errors in this file
                 let Some(diagnostics) = CheckRequest { snap }.request(&mut ctx) else {
@@ -663,9 +664,11 @@ impl CompileHandler<LspCompilerFeat, ProjectInsStateExt> for CompileHandlerImpl
             let Some(compile_fn) = s.may_compile(&c.handler) else {
                 continue;
             };
+            let id = s.snapshot().world().main_id();
 
             s.ext.compiling_since = Some(tinymist_std::time::now());
             spawn_cpu(move || {
+                let _guard = GLOBAL_STATS.stat(id, "main_compile");
                 compile_fn();
             });
         }
