@@ -4,6 +4,7 @@ import { Tracing } from "./features/tracing";
 import { Summary } from "./features/summary";
 import { Diagnostics } from "./features/diagnostics";
 import { Docs } from "./features/docs";
+import Exporter from "./features/exporter";
 import FontView from "./features/font-view";
 
 mainHarness({
@@ -14,4 +15,5 @@ mainHarness({
   diagnostics: Diagnostics,
   "font-view": FontView,
   docs: Docs,
+  exporter: Exporter,
 });
